#include "BLDC.h"
#include "n32g430.h"
#include "string.h"
#include "stdio.h"
#include "math.h"

#define USE_BEMF 1

/**
 * pwm定时器:TIM1  128MHz
 * 消磁定时器:TIM6  64MHz
 * 换相定时器：TIM3  64MHz
 * 霍尔定时器：TIM2  64MHz
 * 
 * 串口：UART4
 * 串口DMA：Tx:CH1   Rx:CH2
 * 
 * I2C：I2C1
 * I2C DMA：：Tx:CH3   Rx:CH4
 */

/**
 * 对高电平有效的栅极驱动，极性永远为P=0(HIGH,高电平有效)，不然会影响到死区电平
 * 如果N接上管，可以用PWM2模式
 * 不管用哪种PWM模式，强制有效/无效时的输出只与极性有关
 * 例如强制有效时，P=0.则CH=1,CHN=0
 * 强制为低：OC1MD=101
 * 强制浮空：EN/P,NEN/NP=0
 */
#define CCMOD_PWM TIM_OCMODE_PWM2
#define CCMOD_LOW TIM_OCMODE_ACTIVE
#define CCMOD_OFF TIM_OCMODE_PWM2
#define CCEN_PWM (TIM_CCEN_CC1EN + TIM_CCEN_CC1NEN)
#define CCEN_LOW (TIM_CCEN_CC1EN + TIM_CCEN_CC1NEN)
#define CCEN_OFF 0

#define COMP_BASIC_CONGIG (COMP1_INMSEL_PA4 + COMP1_CTRL_EN)
#define COMP_A COMP1_INPSEL_PA0
#define COMP_B COMP1_INPSEL_PA2
#define COMP_C COMP1_INPSEL_PA1

#define ACC_STEPS 32  //加速步数
#define ADV_SUBDIV 64  //超前换相细分
#define MIN_DC 384

uint8_t str[128], i2cRecData[10], i2cRecDataIdx = 0;
uint8_t sector = 1;
uint16_t minContEdge = 30, maxErrEdgeCnt = 3;  //30=5us
uint16_t maxDemagTime = 15*64;
bool bemfReady = false, demagFinish = false;
enum MotorState{MotorRun, MotorStop}motorState = MotorStop;
uint16_t commIntervalBegin = 5000, commIntervalEnd = 300;
uint16_t advance = 20;
uint16_t commInterval;
uint16_t throttle = 0;

void mDelay(uint32_t n);

void gpioInit(){
	RCC_AHB_Peripheral_Clock_Enable(RCC_AHB_PERIPH_GPIOA | RCC_AHB_PERIPH_GPIOB |
									RCC_AHB_PERIPH_GPIOC | RCC_AHB_PERIPH_GPIOD);
	RCC_APB2_Peripheral_Clock_Enable(RCC_APB2_PERIPH_AFIO);

	GPIO_InitType gpio;

	//UART Init
	gpio.GPIO_Alternate = GPIO_AF6_UART4;
	gpio.GPIO_Current = GPIO_DS_2MA;
	gpio.GPIO_Mode = GPIO_MODE_AF_PP;
	gpio.GPIO_Pull = GPIO_NO_PULL;
	gpio.GPIO_Slew_Rate = GPIO_SLEW_RATE_SLOW;
	gpio.Pin = GPIO_PIN_12 | GPIO_PIN_13;
	GPIO_Peripheral_Initialize(GPIOD, &gpio);
	GPIO_Peripheral_Initialize(GPIOD, &gpio);

	//TIMER1 Pin Init
	gpio.GPIO_Alternate = GPIO_AF3_TIM1;
	gpio.GPIO_Slew_Rate = GPIO_SLEW_RATE_FAST;
	gpio.Pin = GPIO_PIN_8 | GPIO_PIN_9 | GPIO_PIN_10;  //CH1/2/3
	GPIO_Peripheral_Initialize(GPIOA, &gpio);
	gpio.GPIO_Alternate = GPIO_AF2_TIM1;
	gpio.Pin = GPIO_PIN_13;  //CH1N
	GPIO_Peripheral_Initialize(GPIOC, &gpio);
	gpio.GPIO_Alternate = GPIO_AF8_TIM1;
	gpio.Pin = GPIO_PIN_15;  //CH2N
	GPIO_Peripheral_Initialize(GPIOD, &gpio);
	gpio.GPIO_Alternate = GPIO_AF2_TIM1;
	gpio.Pin = GPIO_PIN_14;  //CH3N
	GPIO_Peripheral_Initialize(GPIOD, &gpio);

	//TIMER8 Test
	gpio.GPIO_Alternate = GPIO_AF13_TIM8;
	gpio.GPIO_Slew_Rate = GPIO_SLEW_RATE_FAST;
	gpio.Pin = GPIO_PIN_14;  //CH1
	GPIO_Peripheral_Initialize(GPIOB, &gpio);
	gpio.GPIO_Alternate = GPIO_AF10_TIM8;
	gpio.Pin = GPIO_PIN_3;  //CH2
	GPIO_Peripheral_Initialize(GPIOA, &gpio);
	gpio.GPIO_Alternate = GPIO_AF10_TIM8;
	gpio.Pin = GPIO_PIN_4;  //CH3
	GPIO_Peripheral_Initialize(GPIOB, &gpio);
	gpio.GPIO_Alternate = GPIO_AF8_TIM8;
	gpio.Pin = GPIO_PIN_15;  //CH1N
	GPIO_Peripheral_Initialize(GPIOB, &gpio);
	gpio.GPIO_Alternate = GPIO_AF8_TIM8;
	gpio.Pin = GPIO_PIN_0;  //CH2N
	GPIO_Peripheral_Initialize(GPIOB, &gpio);
	gpio.GPIO_Alternate = GPIO_AF13_TIM8;
	gpio.Pin = GPIO_PIN_6;  //CH3N
	GPIO_Peripheral_Initialize(GPIOB, &gpio);

	//TIMER2 HALL Init
	gpio.GPIO_Mode = GPIO_MODE_INPUT;
	gpio.GPIO_Pull = GPIO_PULL_UP;
	gpio.GPIO_Alternate = GPIO_AF6_TIM2;
	gpio.Pin = GPIO_PIN_15;
	GPIO_Peripheral_Initialize(GPIOA, &gpio);
	gpio.GPIO_Alternate = GPIO_AF3_TIM2;
	gpio.Pin = GPIO_PIN_3 | GPIO_PIN_10;
	GPIO_Peripheral_Initialize(GPIOB, &gpio);

	//COMP Init
	gpio.GPIO_Mode = GPIO_MODE_ANALOG;
	gpio.GPIO_Current = GPIO_DS_4MA;
	gpio.GPIO_Pull = GPIO_NO_PULL;
	gpio.Pin = GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_4;
	GPIO_Peripheral_Initialize(GPIOA, &gpio);
	gpio.GPIO_Mode = GPIO_MODE_AF_PP;
	gpio.Pin = GPIO_PIN_11;
	gpio.GPIO_Alternate = GPIO_AF8_COMP1;
	GPIO_Peripheral_Initialize(GPIOA, &gpio);

	//I2C Init
	gpio.GPIO_Mode = GPIO_MODE_AF_OD;
	gpio.GPIO_Pull = GPIO_NO_PULL;
	gpio.GPIO_Alternate = GPIO_AF5_I2C1;
	gpio.GPIO_Current = GPIO_DS_12MA;
	gpio.GPIO_Slew_Rate = GPIO_SLEW_RATE_FAST;
	gpio.Pin = GPIO_PIN_8 | GPIO_PIN_9;
	GPIO_Peripheral_Initialize(GPIOB, &gpio);

	//Test Pin Init
	gpio.GPIO_Mode = GPIO_MODE_OUT_PP;
	gpio.Pin = GPIO_PIN_3 | GPIO_PIN_5;
	gpio.GPIO_Alternate = 0;
	GPIO_Peripheral_Initialize(GPIOA, &gpio);
}

void uartInit(){
	RCC_AHB_Peripheral_Clock_Enable(RCC_AHB_PERIPH_DMA);
	RCC_APB2_Peripheral_Clock_Enable(RCC_APB2_PERIPH_UART4);

	//UART DMA Init 
	DMA_InitType dma;
	dma.PeriphAddr     = (uint32_t)&UART4->DAT;
	dma.MemAddr        = (uint32_t)str;
	dma.Direction      = DMA_DIR_PERIPH_DST;
	dma.BufSize        = 1;
	dma.PeriphInc      = DMA_PERIPH_INC_MODE_DISABLE;
	dma.MemoryInc      = DMA_MEM_INC_MODE_ENABLE;
	dma.PeriphDataSize = DMA_PERIPH_DATA_WIDTH_BYTE;
	dma.MemDataSize    = DMA_MEM_DATA_WIDTH_BYTE;
	dma.CircularMode   = DMA_CIRCULAR_MODE_DISABLE;
	dma.Priority       = DMA_CH_PRIORITY_LOW;
	dma.Mem2Mem        = DMA_MEM2MEM_DISABLE;
	DMA_Initializes(DMA_CH1, &dma);
	DMA_Channel_Request_Remap(DMA_CH1, DMA_REMAP_UART4_TX);
	dma.Direction      = DMA_DIR_PERIPH_SRC;
	DMA_Initializes(DMA_CH2, &dma);
	DMA_Channel_Request_Remap(DMA_CH2, DMA_REMAP_UART4_RX);

	//UART Init
	USART_InitType uart;
	uart.HardwareFlowControl = USART_HFCTRL_NONE;
	uart.BaudRate            = 115200;
	uart.Mode                = USART_MODE_RX | USART_MODE_TX;
	uart.Parity              = USART_PE_NO;
	uart.StopBits            = USART_STPB_1;
	uart.WordLength          = USART_WL_8B;
	USART_Initializes(UART4, &uart);

	USART_DMA_Transfer_Enable(UART4, USART_DMAREQ_TX | USART_DMAREQ_RX);

	DMA_Channel_Enable(DMA_CH1);
	DMA_Channel_Enable(DMA_CH2);
	USART_Enable(UART4);
}

void uartSendData(const uint8_t *data, uint16_t length){
	while(!DMA_Flag_Status_Get(DMA, DMA_CH1_TXCF));
	DMA_Flag_Status_Clear(DMA, DMA_CH1_TXCF);
	DMA_Channel_Disable(DMA_CH1);
	DMA_Current_Data_Transfer_Number_Set(DMA_CH1, length);
	DMA_Memory_Address_Config(DMA_CH1, (uint32_t)data);
	DMA_Channel_Enable(DMA_CH1);
}

void PWMTimerInit(){
	RCC_APB2_Peripheral_Clock_Enable(RCC_APB2_PERIPH_TIM1);

	TIM_TimeBaseInitType timBase;
	timBase.Period    = PWM_PERIOD;
	timBase.Prescaler = 0;
	timBase.ClkDiv    = TIM_CLK_DIV1;
	timBase.CntMode   = TIM_CNT_MODE_UP;
	TIM_Base_Initialize(TIM1, &timBase);

	OCInitType oc;
	oc.OutputState  = TIM_OUTPUT_STATE_ENABLE;
	oc.OutputNState = TIM_OUTPUT_NSTATE_ENABLE;
	oc.OcPolarity   = TIM_OC_POLARITY_HIGH;
	oc.OcNPolarity  = TIM_OCN_POLARITY_HIGH;
	oc.OcIdleState  = TIM_OC_IDLE_STATE_RESET;
	oc.OcNIdleState = TIM_OCN_IDLE_STATE_RESET;
	oc.OcMode       = TIM_OCMODE_PWM2;
	oc.Pulse        = 0;
	TIM_Output_Channel1_Initialize(TIM1, &oc);
	TIM_Output_Channel2_Initialize(TIM1, &oc);
	TIM_Output_Channel3_Initialize(TIM1, &oc);

	TIM_BDTRInitType bdtr;
	bdtr.OssrState       = TIM_OSSR_STATE_ENABLE;
	bdtr.OssiState       = TIM_OSSI_STATE_DISABLE;
	bdtr.DeadTime        = 0;
	bdtr.BreakPolarity   = TIM_BREAK_POLARITY_LOW;
	bdtr.AutomaticOutput = TIM_AUTO_OUTPUT_DISABLE;
	bdtr.LockLevel       = TIM_LOCK_LEVEL_OFF;
	bdtr.Break           = TIM_BREAK_IN_DISABLE;
	TIM_Break_And_Dead_Time_Set(TIM1, &bdtr);

	TIM_Capture_Compare_Control_Preload_Enable(TIM1);
	TIM_PWM_Output_Enable(TIM1);
	TIM_On(TIM1);
}

void setPWM(uint16_t cc){
	TIM1->CCDAT1 = cc;
	TIM1->CCDAT2 = cc;
	TIM1->CCDAT3 = cc;
}

void sectorInc(uint8_t *sector){
	if(*sector == 6)
		*sector = 1;
	else
		*sector = *sector + 1;
}

void sectorDec(uint8_t *sector){
	if(*sector == 1)
		*sector = 6;
	else
		*sector = *sector - 1;
}

void setSector(uint8_t sector){
	switch (sector)
	{
	case 1:
		TIM1->CCMOD1 = CCMOD_PWM | CCMOD_OFF << 8;  //A-PWM, B-OFF
		TIM1->CCMOD2 = CCMOD_LOW;  //C-LOW
		TIM1->CCEN   = CCEN_PWM | CCEN_OFF << 4 | CCEN_LOW << 8;

		TIM1->CCDAT2 = 0;
		TIM1->CCDAT3 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_B;
		EXTI->RT_CFG    = EXTI_LINE21;
		EXTI->FT_CFG    = 0;
		break;
	case 2:
		TIM1->CCMOD1 = CCMOD_OFF | CCMOD_PWM << 8;
		TIM1->CCMOD2 = CCMOD_LOW;
		TIM1->CCEN   = CCEN_OFF | CCEN_PWM << 4 | CCEN_LOW << 8;

		TIM1->CCDAT1 = 0;
		TIM1->CCDAT3 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_A;
		EXTI->RT_CFG    = 0;
		EXTI->FT_CFG    = EXTI_LINE21;
		break;
	case 3:
		TIM1->CCMOD1 = CCMOD_LOW | CCMOD_PWM << 8;
		TIM1->CCMOD2 = CCMOD_OFF;
		TIM1->CCEN   = CCEN_LOW | CCEN_PWM << 4 | CCEN_OFF << 8;

		TIM1->CCDAT1 = 0;
		TIM1->CCDAT3 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_C;
		EXTI->RT_CFG    = EXTI_LINE21;
		EXTI->FT_CFG    = 0;
		break;
	case 4:
		TIM1->CCMOD1 = CCMOD_LOW | CCMOD_OFF << 8;
		TIM1->CCMOD2 = CCMOD_PWM;
		TIM1->CCEN   = CCEN_LOW | CCEN_OFF << 4 | CCEN_PWM << 8;

		TIM1->CCDAT1 = 0;
		TIM1->CCDAT2 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_B;
		EXTI->RT_CFG    = 0;
		EXTI->FT_CFG    = EXTI_LINE21;
		break;
	case 5:
		TIM1->CCMOD1 = CCMOD_OFF | CCMOD_LOW << 8;
		TIM1->CCMOD2 = CCMOD_PWM;
		TIM1->CCEN   = CCEN_OFF | CCEN_LOW << 4 | CCEN_PWM << 8;

		TIM1->CCDAT1 = 0;
		TIM1->CCDAT2 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_A;
		EXTI->RT_CFG    = EXTI_LINE21;
		EXTI->FT_CFG    = 0;
		break;
	case 6:
		TIM1->CCMOD1 = CCMOD_PWM | CCMOD_LOW << 8;
		TIM1->CCMOD2 = CCMOD_OFF;
		TIM1->CCEN   = CCEN_PWM | CCEN_LOW << 4 | CCEN_OFF << 8;

		TIM1->CCDAT2 = 0;
		TIM1->CCDAT3 = 0;

		COMP->Cmp1.CTRL = COMP_BASIC_CONGIG | COMP_C;
		EXTI->RT_CFG    = 0;
		EXTI->FT_CFG    = EXTI_LINE21;
		break;
	default:
		break;
	}
}

void hallInit(){
	RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_TIM2);

	TIM_TimeBaseInitType timBase;
	timBase.Period    = 0xFFFF;
	timBase.Prescaler = 0;
	timBase.ClkDiv    = TIM_CLK_DIV1;
	timBase.CntMode   = TIM_CNT_MODE_UP;
	timBase.CapCh1Sel = 0;
	timBase.CapCh2Sel = 0;
	timBase.CapCh3Sel = 0;
	timBase.CapCh4Sel = 0;
	timBase.CapEtrClrFromCompEn = 0;
	timBase.CapEtrSelFromTscEn = 0;
	TIM_Base_Initialize(TIM2, &timBase);

	TIM_ICInitType ic;
	ic.IcFilter    = 0;
	ic.IcPolarity  = TIM_IC_POLARITY_RISING;
	ic.IcPrescaler = 0;
	ic.IcSelection = TIM_IC_SELECTION_TRC;
	ic.Channel     = TIM_CH_1;
	TIM_Input_Channel_Initialize(TIM2, &ic);
	TIM_Trigger_Source_Select(TIM2, TIM_TRIG_SEL_TI1F_ED);
	TIM_Hall_Sensor_Enable(TIM2);
	TIM_Slave_Mode_Select(TIM2, TIM_SLAVE_MODE_RESET);

	OCInitType oc;
	oc.OutputState  = TIM_OUTPUT_STATE_DISABLE;
	oc.OutputNState = TIM_OUTPUT_NSTATE_DISABLE;
	oc.OcPolarity   = TIM_OC_POLARITY_HIGH;
	oc.OcNPolarity  = TIM_OCN_POLARITY_HIGH;
	oc.OcIdleState  = TIM_OC_IDLE_STATE_RESET;
	oc.OcNIdleState = TIM_OCN_IDLE_STATE_RESET;
	oc.OcMode       = TIM_OCMODE_PWM1;
	oc.Pulse        = 0;
	TIM_Output_Channel2_Initialize(TIM2, &oc);
	TIM_Output_Trigger_Select(TIM1, TIM_TRGO_SRC_OC2REF);

	NVIC_InitType nvic;
	nvic.NVIC_IRQChannel = TIM2_IRQn;
	nvic.NVIC_IRQChannelCmd = ENABLE;
	nvic.NVIC_IRQChannelPreemptionPriority = 0;
	nvic.NVIC_IRQChannelSubPriority = 0;
	NVIC_Initializes(&nvic);
	TIM_Interrupt_Enable(TIM2, TIM_INT_UPDATE);

	TIM_On(TIM2);
}

void compInit(){
	RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_COMP);

	COMP->Cmp1.CTRL = COMP_BASIC_CONGIG + COMP_A;

	NVIC_InitType nvic;
	nvic.NVIC_IRQChannel = COMP_1_2_3_IRQn;
	nvic.NVIC_IRQChannelCmd = ENABLE;
	nvic.NVIC_IRQChannelPreemptionPriority = 0;
	nvic.NVIC_IRQChannelSubPriority = 0;
	NVIC_Initializes(&nvic);

	COMP->INTEN = COMP1_INTEN;
	EXTI->IMASK = 0;  //不开比较器中断
	EXTI->RT_CFG = EXTI_LINE21;
}

void demagTimerInit(){
	RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_TIM6);

	TIM_TimeBaseInitType timBase;
	timBase.Period = 0xFFFF;
	timBase.Prescaler = 0;
	timBase.ClkDiv = TIM_CLK_DIV1;
	timBase.CntMode = TIM_CNT_MODE_UP;
	TIM_Base_Initialize(TIM6, &timBase);
}

void commTimerInit(){
	RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_TIM3);

	TIM_TimeBaseInitType timBase;
	timBase.Period = commIntervalBegin;
	timBase.Prescaler = 0xFF;
	timBase.ClkDiv = TIM_CLK_DIV1;
	timBase.CntMode = TIM_CNT_MODE_UP;
	TIM_Base_Initialize(TIM3, &timBase);
	TIM3->STS = 0;  //?N32G430会置位更新中断

	NVIC_InitType nvic;
	nvic.NVIC_IRQChannel = TIM3_IRQn;
	nvic.NVIC_IRQChannelCmd = ENABLE;
	nvic.NVIC_IRQChannelPreemptionPriority = 0;
	nvic.NVIC_IRQChannelSubPriority = 0;
	NVIC_Initializes(&nvic);
	TIM_Interrupt_Enable(TIM3, TIM_INT_UPDATE);
}

void i2cInit(){
	RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_I2C1);

	I2C_InitType i2c;
    i2c.BusMode   = I2C_BUSMODE_I2C;
    i2c.DutyCycle = I2C_FMDUTYCYCLE_2;  
    i2c.OwnAddr1  = 0x52;
    i2c.AckEnable = I2C_ACKEN;
    i2c.AddrMode  = I2C_ADDR_MODE_7BIT;
    i2c.ClkSpeed  = 1000000; /* 1000k */
	I2C_Initializes(I2C1, &i2c);

	NVIC_InitType nvic;
	nvic.NVIC_IRQChannel = I2C1_EV_IRQn;
	nvic.NVIC_IRQChannelCmd = ENABLE;
	nvic.NVIC_IRQChannelPreemptionPriority = 0;
	nvic.NVIC_IRQChannelSubPriority = 1;
	NVIC_Initializes(&nvic);
	I2C_Interrupts_Enable(I2C1, I2C_INT_EVENT | I2C_INT_BUF);

	I2C_ON(I2C1);
}

void init(){
	gpioInit();
	uartInit();
	PWMTimerInit();
	i2cInit();

#if USE_BEMF
	compInit();
	demagTimerInit();
	commTimerInit();
#else
	hallInit();
#endif
}

int getPos(){
	int a,b,c;
	a = GPIO_Input_Pin_Data_Get(GPIOA, GPIO_PIN_15);
	b = GPIO_Input_Pin_Data_Get(GPIOB, GPIO_PIN_3);
	c = GPIO_Input_Pin_Data_Get(GPIOB, GPIO_PIN_10);
	return (c << 2) + (b << 1) + a;
}
int8_t posBias = 0;
void hallTimerIntProc(){
	TIM_Flag_Clear(TIM2, TIM_FLAG_UPDATE);

	switch(getPos()){
	case 4:
		sector = (1 + posBias)%6 + 1;
		break;
	case 6:
		sector = (2 + posBias)%6 + 1;
		break;
	case 2:
		sector = (3 + posBias)%6 + 1;
		break;
	case 3:
		sector = (4 + posBias)%6 + 1;
		break;
	case 1:
		sector = (5 + posBias)%6 + 1;
		break;
	case 5:
		sector = (6 + posBias)%6 + 1;
		break;
	
	default:
		break;
	}
	setSector(sector);
	TIM_Event_Generate(TIM1, TIM_EVT_SRC_COM);
	setPWM(throttle);
}

void compIntProc(){
	uint16_t cntNow = TIM3->CNT;
	int errCnt = 0, errContCnt = 0, zc = 0, corrCnt = 0;

	if(demagFinish == false){
		if(TIM6->CNT < maxDemagTime){
			EXTI->PEND = EXTI_LINE21;  //记得清中断
			return;
		} else {
			demagFinish = true;
		}
	}
	if(sector & 0x01){
		//1 3 5 Rising Edge
		for (int i = 0; i < minContEdge; i++){
			if(!(COMP->Cmp1.CTRL & COMP1_CTRL_OUT)){
				errCnt++;
				errContCnt++;
			} else {
				corrCnt++;
				errContCnt = 0;
			}

			//太多错误连续出现直接跳出
			if(errContCnt >= maxErrEdgeCnt){
				corrCnt = 0;
				break;
			}
		}
	} else {
		//2 4 6 Falling Edge
		for (int i = 0; i < minContEdge; i++){
			if(COMP->Cmp1.CTRL & COMP1_CTRL_OUT){
				errCnt++;
				errContCnt++;
			} else {
				corrCnt++;
				errCnt = 0;
			}

			if(errContCnt >= maxErrEdgeCnt){
				corrCnt = 0;
				break;
			}
		}
	}
	if(corrCnt >= errCnt*2){
		zc = 1;
	}

	if(zc){
		uint16_t div = TIM3->PSC;
		
		commInterval = (cntNow + commInterval)/2;
		
		//自动缩放
		if(commInterval < 10000 && div != 0){
			div /= 2;
			commInterval *= 2;
			TIM3->PSC = div;
		} else if(commInterval > 40000 && div != 0xFFFF){
			div = div*2 + 1;
			commInterval /= 2;
			TIM3->PSC = div;
		}

		TIM3->CNT -= cntNow;  //保证精度
		TIM3->AR = commInterval - commInterval/ADV_SUBDIV*advance;  //-超前换相

		EXTI->IMASK = 0;  //关比较器中断

		if(sector & 0x01)
			GPIO_Pins_Set(GPIOA, GPIO_PIN_5);
		else
			GPIO_Pins_Reset(GPIOA, GPIO_PIN_5);
	}

	EXTI->PEND = EXTI_LINE21;
}

void commTimerIntProc(){
	if(bemfReady == false){
		uint16_t arr = TIM3->AR;
		if(arr < commIntervalEnd){
			commInterval = arr/2;
			bemfReady = true;

			sectorInc(&sector);
			setSector(sector);
			TIM_Event_Generate(TIM1, TIM_EVT_SRC_COM);
			setPWM(throttle);

			TIM3->AR = 0xFFFF;  //记录换相延时
			TIM3->CNT = 0;

			TIM6->CNT = 0;  //记录消磁延时
			demagFinish = false;

			EXTI->IMASK = EXTI_LINE21;;  //开比较器中断
		} else {
			arr -= arr/ACC_STEPS;
			TIM3->AR = arr;

			sectorInc(&sector);
			setSector(sector);
			TIM_Event_Generate(TIM1, TIM_EVT_SRC_COM);
			setPWM(throttle);
		}
	} else {
		sectorInc(&sector);
		setSector(sector);
		TIM_Event_Generate(TIM1, TIM_EVT_SRC_COM);
		setPWM(throttle);

		TIM3->AR = 0xFFFF;  //记录换相延时
		TIM3->CNT = 0;

		TIM6->CNT = 0;  //记录消磁延时
		demagFinish = false;

		EXTI->IMASK = EXTI_LINE21;;  //开比较器中断
	}

	TIM3->STS = 0;
}

void startup(){
	setSector(sector);
	TIM_Event_Generate(TIM1, TIM_EVT_SRC_COM);
	setPWM(throttle);

	//换相定时器
	TIM3->CTRL1 = TIM_CTRL1_CNTEN;

	//消磁定时器
	TIM6->CTRL1 = TIM_CTRL1_CNTEN;
}

void motorDeinit(){
	setPWM(0);

	TIM3->CTRL1 = 0;
	TIM3->PSC = 0xFF;
	TIM3->AR = commIntervalBegin;
	TIM3->CNT = 0;

	EXTI->IMASK = 0;

	bemfReady = false;
	demagFinish = false;
}

void i2cEventIntProc(){
	uint16_t sts1 = I2C1->STS1;
	if(sts1 & I2C_STS1_ADDRF){
		i2cRecDataIdx = 0;
		I2C1->STS2;
		return;
	}
	if(sts1 & I2C_STS1_RXDATNE){
		if(i2cRecDataIdx == 10)
			while(1);
		i2cRecData[i2cRecDataIdx++] = I2C1->DAT;
		return;
	}
	if(sts1 & I2C_STS1_STOPF){
		I2C1->CTRL1 = I2C_CTRL1_EN | I2C_CTRL1_ACKEN;

		if((uint8_t)(i2cRecData[0] + i2cRecData[1]) == i2cRecData[2]){
			throttle = (i2cRecData[0] << 8) + i2cRecData[1];

			//上下限
			if(throttle > PWM_PERIOD)
				throttle = PWM_PERIOD;
			else if(throttle < MIN_DC)
				throttle = 0;

			switch(motorState){
			case MotorRun:
				if(!throttle){
					motorState = MotorStop;
					motorDeinit();
				}
			break;
			case MotorStop:
				if(throttle){
					startup();
					motorState = MotorRun;
				}
			break;
			}
		}
		return;
	}
}

void run(){
	init();

	while(1){
	}
}